;-- TIME_STAMP   2019-01-12 16:45:25   v 0.1

#Region ;************ Includes ************
#include-once
#include <Array.au3>
#include <WinAPIShellEx.au3>
#EndRegion ;************ Includes ************

Global $g_iSHNotifyMsg, $g_iSHNotifyID, $g_aSHNotifyPaths[0], $g_aSHNotifyExts[0]

Func _ShellChangeNotify($hWnd, $vNotify)
	Local $aPaths[0][2], $aSplit

	; Message-Handler für 'WM_SHELLCHANGENOTIFY' registrieren, damit Windows uns diese Nachrichten sendet.
	If Not $g_iSHNotifyMsg Then
		OnAutoItExitRegister('_ShellChangeNotifyExit')
		$g_iSHNotifyMsg = _WinAPI_RegisterWindowMessage('SHELLCHANGENOTIFY')
		GUIRegisterMsg($g_iSHNotifyMsg, 'WM_SHELLCHANGENOTIFY')
		ConsoleWrite('> ShellChangeNotify wurde aktiviert!' & @CRLF)
	EndIf

	Select
		Case IsArray($vNotify)
			If UBound($vNotify, 2) <> 2 Then
				MsgBox(BitOR($MB_ICONERROR, $MB_SYSTEMMODAL), 'Error', '_ShellChangeNotify: $vNotify must have 2 dimensions!')
				Return SetError(1, 0, False)
			EndIf
			If UBound($vNotify) = 0 Then
				_ShellChangeNotifyExit()
				Return True
			EndIf
			$aPaths = $vNotify
		Case $vNotify = ''
			_ShellChangeNotifyExit()
			Return True
		Case Else
			ReDim $aPaths[1][2]
			$aPaths[0][0] = $vNotify
			$aPaths[0][1] = '*.*'
	EndSelect

	Local $e, $j, $aExts[0]
	ReDim $g_aSHNotifyPaths[0]
	ReDim $g_aSHNotifyExts[0]
	For $i = 0 To UBound($aPaths) - 1 Step 1
		ReDim $aExts[0]
		If StringInStr($aPaths[$i][1], '|') Then
			$aSplit = StringSplit($aPaths[$i][1], '|', 2)
			For $s = 0 To UBound($aSplit) -1 Step 1
				_ArrayAdd($aExts, $aSplit[$s])
			Next
		Else
			_ArrayAdd($aExts, $aPaths[$i][1])
		EndIf

		If StringInStr($aPaths[$i][0], '|') Then
			$aSplit = StringSplit($aPaths[$i][0], '|', 2)
			For $s = 0 To UBound($aSplit) -1 Step 1
				$j = _ArrayAdd($g_aSHNotifyPaths, $aSplit[$s])
				If Not FileExists($g_aSHNotifyPaths[$j]) Then
					MsgBox(BitOR($MB_ICONERROR, $MB_SYSTEMMODAL), 'Error', '_ShellChangeNotify: Verzeichnis nicht gefunden: ' & $g_aSHNotifyPaths[$j])
					Return SetError(2, $i, False)
				EndIf
			Next
		Else
			$j = _ArrayAdd($g_aSHNotifyPaths, $aPaths[$i][0])
			If Not FileExists($g_aSHNotifyPaths[$j]) Then
				MsgBox(BitOR($MB_ICONERROR, $MB_SYSTEMMODAL), 'Error', '_ShellChangeNotify: Verzeichnis nicht gefunden: ' & $g_aSHNotifyPaths[$j])
				Return SetError(2, $i, False)
			EndIf
		EndIf
		$e = _ArrayAdd($g_aSHNotifyExts, '')
		$g_aSHNotifyExts[$e] = $aExts
	Next
	ConsoleWrite('> Verzeichnisse, die überwacht werden:' & @CRLF)
	ConsoleWrite('> ' & @TAB & _ArrayToString($g_aSHNotifyPaths, ', ') & @CRLF)
	ConsoleWrite('> Dateierweiterungen, die überwacht werden:' & @CRLF)
	Local $aExts[0]
	For $i = 0 To UBound($g_aSHNotifyExts) -1 Step 1
		_ArrayAdd($aExts, $g_aSHNotifyExts[$i])
	Next
	$aExts = _ArrayUnique($aExts, 0, 0, 0, 0)
		ConsoleWrite('> ' & @TAB & _ArrayToString($aExts, ', ') & @CRLF)

	$g_iSHNotifyID = _WinAPI_ShellChangeNotifyRegister($hWnd, $g_iSHNotifyMsg, $SHCNE_ALLEVENTS, BitOR($SHCNRF_INTERRUPTLEVEL, $SHCNRF_SHELLLEVEL, $SHCNRF_RECURSIVEINTERRUPT), $g_aSHNotifyPaths, 1) ; alle Nachrichten, rekursiv
	If @error Then
		Local $iError = @error
		MsgBox(BitOR($MB_ICONERROR, $MB_SYSTEMMODAL), 'Error', '_ShellChangeNotify: Window does not registered.')
		Return SetError(3, $iError, False)
	EndIf
EndFunc   ;==>_ShellChangeNotify

Func WM_SHELLCHANGENOTIFY($hWnd, $iMsg, $wParam, $lParam)
	#forceref $hWnd, $iMsg
	Local $sFilePath = _WinAPI_ShellGetPathFromIDList(DllStructGetData(DllStructCreate('dword Item1; dword Item2', $wParam), 'Item1'))
	If $sFilePath Then
		Local $sPath = StringRegExpReplace($sFilePath, '(.+)\\.+', '\1'), $sFile = StringRegExpReplace($sFilePath, '.+\\(.+)', '\1'), $sExt = StringRegExpReplace($sFilePath, '.+(\..+$)', '\1')
		Local $iPath = _ArraySearch($g_aSHNotifyPaths, $sPath)
		Local $iExt = $iPath > -1 ? _ArraySearch($g_aSHNotifyExts[$iPath], $sExt) : -1
		Local $sMsg = ($iPath = -1 ? '! Der Pfad ('&$sPath&') interessiert uns aber nicht!' : '+ Der Pfad ('&$sPath&') interessiert uns!') & @CRLF
		$sMsg &= ($iExt = -1 ? '! Doch die Erweiterung ('&$sExt&') interessiert uns nicht!' : '+ Die Erweiterung ('&$sExt&') interessiert uns auch!') & @CRLF
		Switch $lParam
			Case $SHCNE_ALLEVENTS        ; * 0x7FFFFFFF Alle Ereignisse sind aufgetreten.
			Case $SHCNE_ASSOCCHANGED     ;   0x08000000 Eine Dateitypzuordnung hat sich geändert.
			Case $SHCNE_ATTRIBUTES       ;   0x00000800 Die Attribute eines Elements oder Ordners haben sich geändert.
			Case $SHCNE_CREATE           ;   0x00000002 Es wurde ein Nicht-Ordnerelement erstellt.
				ConsoleWrite('+ WM_SHELLCHANGENOTIFY: $SHCNE_CREATE           | Es wurde ein Nicht-Ordnerelement erstellt.' & @CRLF)
				ConsoleWrite('+ WM_SHELLCHANGENOTIFY: Event: 0x' & Hex($lParam) & '       | Path: ' & $sFilePath & @CRLF)
				ConsoleWrite($sMsg)
				If $iPath > -1 And $iExt > -1 Then
					ConsoleWrite("> $sPath --> " & $sPath & @CRLF)
					ConsoleWrite("> $sFile --> " & $sFile & @CRLF)
					ConsoleWrite("> $sExt  --> " & $sExt & @CRLF)
					ConsoleWrite('> Mach was damit...' & @CRLF)
				EndIf
				ConsoleWrite('- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -' & @CRLF)
			Case $SHCNE_DELETE           ;   0x00000004 Ein Nicht-Ordnerelement wurde gelöscht.
				ConsoleWrite('> WM_SHELLCHANGENOTIFY: $SHCNE_DELETE           | Ein Nicht-Ordnerelement wurde gelöscht.' & @CRLF)
				ConsoleWrite('> WM_SHELLCHANGENOTIFY: Event: 0x' & Hex($lParam) & '       | Path: ' & $sFilePath & @CRLF)
				ConsoleWrite($sMsg)
				If $iPath > -1 And $iExt > -1 Then
					ConsoleWrite("> $sPath --> " & $sPath & @CRLF)
					ConsoleWrite("> $sFile --> " & $sFile & @CRLF)
					ConsoleWrite("> $sExt  --> " & $sExt & @CRLF)
					ConsoleWrite('> Mach was damit...' & @CRLF)
				EndIf
				ConsoleWrite('- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -' & @CRLF)
			Case $SHCNE_DRIVEADD         ;   0x00000100 Ein Laufwerk wurde hinzugefügt.
			Case $SHCNE_DRIVEADDGUI      ; * 0x00010000 Windows XP und höher: Wird nicht verwendet.
			Case $SHCNE_DRIVEREMOVED     ;   0x00000080 Ein Laufwerk wurde entfernt.
			Case $SHCNE_EXTENDED_EVENT   ; * 0x04000000 Derzeit nicht verwendet.
				ConsoleWrite('+ WM_SHELLCHANGENOTIFY: $SHCNE_EXTENDED_EVENT   | Derzeit nicht verwendet.' & @CRLF)
				ConsoleWrite('+ WM_SHELLCHANGENOTIFY: Event: 0x' & Hex($lParam) & '       | Path: ' & $sFilePath & @CRLF)
				ConsoleWrite('- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -' & @CRLF)
			Case $SHCNE_FREESPACE        ;   0x00040000 Der freie Speicherplatz auf einem Laufwerk hat sich geändert.
				ConsoleWrite('+ WM_SHELLCHANGENOTIFY: $SHCNE_FREESPACE        | Es wurde ein Nicht-Ordnerelement erstellt.' & @CRLF)
				ConsoleWrite('+ WM_SHELLCHANGENOTIFY: Event: 0x' & Hex($lParam) & '       | Path: ' & $sFilePath & @CRLF)
				ConsoleWrite('- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -' & @CRLF)
			Case $SHCNE_MEDIAINSERTED    ;   0x00000020 Speichermedium wurde in ein Laufwerk eingelegt.
			Case $SHCNE_MEDIAREMOVED     ;   0x00000040 Speichermedien wurden aus einem Laufwerk entfernt.
			Case $SHCNE_MKDIR            ;   0x00000008 Ein Ordner wurde erstellt.
			Case $SHCNE_NETSHARE         ;   0x00000200 Ein Ordner auf dem lokalen Computer wird über das Netzwerk freigegeben.
			Case $SHCNE_NETUNSHARE       ;   0x00000400 Ein Ordner auf dem lokalen Computer wird nicht mehr über das Netzwerk freigegeben.
			Case $SHCNE_RENAMEFOLDER     ;   0x00020000 Der Name eines Ordners hat sich geändert.
			Case $SHCNE_RENAMEITEM       ;   0x00000001 Der Name eines Nicht-Ordnerelements wurde geändert.
			Case $SHCNE_RMDIR            ;   0x00000010 Ein Ordner wurde entfernt.
			Case $SHCNE_SERVERDISCONNECT ;   0x00004000 Der Computer hat die Verbindung zu einem Server getrennt.
			Case $SHCNE_UPDATEDIR        ;   0x00001000 Der Inhalt eines vorhandenen Ordners hat sich geändert, der Ordner ist jedoch noch vorhanden und wurde nicht umbenannt.
				ConsoleWrite('+ WM_SHELLCHANGENOTIFY: $SHCNE_UPDATEDIR        | Der Inhalt eines vorhandenen Ordners hat sich geändert, der Ordner ist jedoch noch vorhanden und wurde nicht umbenannt.' & @CRLF)
				ConsoleWrite('+ WM_SHELLCHANGENOTIFY: Event: 0x' & Hex($lParam) & '       | Path: ' & $sFilePath & @CRLF)
				ConsoleWrite('- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -' & @CRLF)
			Case $SHCNE_UPDATEIMAGE      ;   0x00008000 Ein Bild in der Systemabbildliste wurde geändert.
			Case $SHCNE_UPDATEITEM       ;   0x00002000 Ein vorhandenes Element (ein Ordner oder ein Nichtordner) hat sich geändert, das Element ist jedoch noch vorhanden und wurde nicht umbenannt.
				ConsoleWrite('+ WM_SHELLCHANGENOTIFY: $SHCNE_UPDATEITEM       | Ein vorhandenes Element (ein Ordner oder ein Nichtordner) hat sich geändert, das Element ist jedoch noch vorhanden und wurde nicht umbenannt.' & @CRLF)
				ConsoleWrite('+ WM_SHELLCHANGENOTIFY: Event: 0x' & Hex($lParam) & '       | Path: ' & $sFilePath & @CRLF)
				ConsoleWrite('- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -' & @CRLF)
			Case $SHCNE_DISKEVENTS       ; * 0x0002381F Gibt eine Kombination aller Plattenereignis-IDs an.
			Case $SHCNE_GLOBALEVENTS     ; * 0x0C0581E0 Gibt eine Kombination aller globalen Ereignisbezeichner an.
			Case $SHCNE_INTERRUPT        ; * 0x80000000 Das angegebene Ereignis ist infolge eines Systeminterrupts aufgetreten. Da dieser Wert andere Ereigniswerte ändert, kann er nicht alleine verwendet werden.
		EndSwitch
	Else
		ConsoleWrite('- WM_SHELLCHANGENOTIFY: Event: 0x' & Hex($lParam) & @CRLF)
		ConsoleWrite('- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -' & @CRLF)
	EndIf
EndFunc   ;==>WM_SHELLCHANGENOTIFY

Func _ShellChangeNotifyExit()
	If $g_iSHNotifyID Then
		ConsoleWrite('> Deregister($g_iSHNotifyID) --> ' & $g_iSHNotifyID & @CRLF)
		_WinAPI_ShellChangeNotifyDeregister($g_iSHNotifyID)
		$g_iSHNotifyID = 0
	EndIf
EndFunc   ;==>_ShellChangeNotifyExit
